#include "stdafx.h"
#include <vector>
#include <Python.h>
#include "WiimoteManager.h"

static PyObject* WiimoteAPI_Update(PyObject* self, PyObject* args)
{
	WiimoteManager* wiimoteManager = (WiimoteManager*)self;
	int nbConnected = wiimoteManager->getNbConnected();

	if(nbConnected == 0)
		Py_RETURN_NONE;

	wiimoteManager->update();

	std::vector<PyObject*> vecStates;

	for(int i=0; i<nbConnected; i++)
	{
		WiimoteButtonsStates* btnsStates = wiimoteManager->getButtonsStates(i);
		int stA		=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::A));
		int stB		=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::B));
		int stMinus	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Minus));
		int stPlus	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Plus));
		int stHome	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Home));
		int stOne	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::One));
		int stTwo	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Two));
		int stUp	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Up));
		int stDown	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Down));
		int stLeft	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Left));
		int stRight	=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Right));
		int stZ		=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::Z));
		int stC		=	(int)(btnsStates->getButtonState(WiimoteButtonsStates::C));

		float xNunchuk = wiimoteManager->getWiimote(i)->Nunchuk.Joystick.X;
		float yNunchuk = wiimoteManager->getWiimote(i)->Nunchuk.Joystick.Y;

		bool dot0Visible = wiimoteManager->getWiimote(i)->IR.Dot[0].bVisible;
		bool dot1Visible = wiimoteManager->getWiimote(i)->IR.Dot[1].bVisible;
		
		float xCursor = wiimoteManager->getCursorPosition(i,0);
		float yCursor = wiimoteManager->getCursorPosition(i,1);

		float pitch = wiimoteManager->getWiimote(i)->Acceleration.Orientation.Pitch;
		float roll = wiimoteManager->getWiimote(i)->Acceleration.Orientation.Roll;

		float accelX = wiimoteManager->getWiimote(i)->Acceleration.RawX;
		float accelY = wiimoteManager->getWiimote(i)->Acceleration.RawY;
		float accelZ = wiimoteManager->getWiimote(i)->Acceleration.RawZ;
		bool extraX1 =  wiimoteManager->getWiimote(i)->Acceleration.extraX1;
		bool extraX2 = wiimoteManager->getWiimote(i)->Acceleration.extraX2;
		bool extraY = wiimoteManager->getWiimote(i)->Acceleration.extraY;
		bool extraZ = wiimoteManager->getWiimote(i)->Acceleration.extraZ;

		vecStates.push_back(Py_BuildValue(	"[i,i,i,i,i,i,i,i,i,i,i,i,i,f,f,b,b,f,f,f,f,f,f,f,b,b,b,b]",
											stA,
											stB,
											stMinus,
											stPlus,
											stHome,
											stOne,
											stTwo,
											stUp,
											stDown,
											stLeft,
											stRight,
											stZ,
											stC,
											xNunchuk,
											yNunchuk,
											dot0Visible,
											dot1Visible,
											xCursor,
											yCursor,
											pitch,
											roll,
											accelX,
											accelY,
											accelZ,
											extraX1,
											extraX2,
											extraY,
											extraZ));
	}

	PyObject* states;

	if(nbConnected == 1)
		states = Py_BuildValue("[O]",vecStates[0]);
	else if(nbConnected == 2)
		states = Py_BuildValue("[OO]",vecStates[0],vecStates[1]);

	PyObject* res = Py_BuildValue("[iO]",nbConnected,states);
    return res;
}

static PyObject* WiimoteAPI_Rumble(PyObject* self, PyObject* args)
{
	int id_wiimote, duration = 0;
	PyArg_ParseTuple(args,"ii",&id_wiimote,&duration);
	WiimoteManager* wiimoteManager = (WiimoteManager*)self;
	wiimoteManager->getWiimote(id_wiimote)->RumbleForAsync(duration);
	PyObject* res = Py_BuildValue("i",0);
	return res;
}

static PyObject* WiimoteAPI_PlaySample(PyObject* self, PyObject* args)
{
	int id_wiimote = 0;
	PyArg_ParseTuple(args,"i",&id_wiimote);
	WiimoteManager* wiimoteManager = (WiimoteManager*)self;
	wiimote_sample masterSample = wiimoteManager->getMasterSample();
	wiimoteManager->getWiimote(id_wiimote)->PlaySample(masterSample);
	PyObject* res = Py_BuildValue("i",0);
	return res;
}

static PyMethodDef WiimoteAPIMethods[] = {
    {"Update", WiimoteAPI_Update, METH_VARARGS, "Get the values from the wiimotes"},
    {"Rumble", WiimoteAPI_Rumble, METH_VARARGS, "Rumble the wiimotes"},
    {"PlaySample", WiimoteAPI_PlaySample, METH_VARARGS, "Play sample on the wiimotes"},
    {NULL, NULL, 0, NULL}     /* Always use this as the last line in your table. */
};


PyMODINIT_FUNC initWiimoteAPI(void)
{
	WiimoteManager* wiimoteManager = new WiimoteManager();
    (void) Py_InitModule4("WiimoteAPI", WiimoteAPIMethods, NULL, (PyObject*)wiimoteManager, PYTHON_API_VERSION);
}


int main(int argc, char* argv[])
{
	Py_SetProgramName(argv[0]);
	Py_Initialize();
	initWiimoteAPI();
	return(EXIT_SUCCESS);
}